function [errp, errv] = sp(N,r,M,maxv,frames,seed,bbcol)
% 2D particle simulator
% N      -> resolution
% r      -> radius of each particle in pixels
% M      -> number of particles
% maxv   -> maximum velocity (displacement per frame) in each direction
% frames -> number of frames to simulate
% seed   -> a random seed used to produce random colors and initial conditions
% bbcol  -> the collision response function used to resolve collisions
rand('seed', seed);

% generate a few particles
p = genparticles(N,r,M,maxv);

% generate a random color for each particle
c = gencolors(M);
f = fillcolorpoints(p, r, N);
I = fillcolorimage(f, N, c);

% assign random initial velocity vectors:
v = 2.0*maxv*(rand(size(p)) - 0.5);

imwrite(I, 'img/imv-00000.png'); % write initial image

% animate
pn = p;
len = length(pn);
collisions = 0;

% gather error information
errp = zeros(1, frames);
errv = zeros(1, frames);

tic;
for t = 1:frames
  pn = round(pn + v);

  % check if two particles intersect, if so change velocity
  for i = 1:(len-1)
    for j = (i+1):len
      if norm(pn(:,i) - pn(:,j)) < r+r-2 % ball-ball collision detected
        collisions = collisions + 1;

        [pos vel] = bbcol(pn(:,i), pn(:,j), v(:,i), v(:,j), r);

        % compute the error as compared to the actual physical simulator
        [pos2 vel2] = bbcollision(pn(:,i), pn(:,j), v(:,i), v(:,j), r);
        errp(collisions) = 0.5*(norm(pos(:,2) - pos2(:,2)) + norm(pos(:,1) - pos2(:,1)));
        errv(collisions) = 0.5*(norm(vel(:,2) - vel2(:,2)) + norm(vel(:,1) - vel2(:,1)));

        pn(:,i) = pos(:,1);
        pn(:,j) = pos(:,2);
        v(:,i) = vel(:,1);
        v(:,j) = vel(:,2);

      end % if ball-ball collision
    end % for each particle j
  end % for each particle i

  [pn v] = wbcollisions(pn, v, N, r);

  fn = fillcolorpoints(pn, r, N);
  In = fillcolorimage(fn, N, c);
  imwrite(In, sprintf('img/imv-%05d.png', t));
end
toc % display elapsed simulation time

errp(collisions+1:end) = [];
errv(collisions+1:end) = [];

collisions % display number of ball-ball collisions
